# Generated by Django 2.0.1 on 2018-01-23 16:32

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0003_auto_20180123_2304'),
    ]

    operations = [
        migrations.RenameField(
            model_name='client',
            old_name='need_auth',
            new_name='auth',
        ),
    ]