# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2018-01-31 04:10
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0004_auto_20180124_0032'),
    ]

    operations = [
        migrations.CreateModel(
            name='Task',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('project', models.CharField(blank=True, max_length=255, null=True)),
                ('spider', models.CharField(blank=True, max_length=255, null=True)),
                ('name', models.CharField(blank=True, max_length=255, null=True)),
                ('args', models.TextField(blank=True, null=True)),
                ('description', models.TextField(blank=True, null=True)),
                ('success', models.IntegerField(blank=True, default=0)),
                ('error', models.IntegerField(blank=True, default=0)),
                ('last', models.DateTimeField(blank=True, null=True)),
                ('configuration', models.TextField(blank=True, null=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('client', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, to='core.Client')),
            ],
        ),
        migrations.AlterField(
            model_name='monitor',
            name='configuration',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='monitor',
            name='description',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='monitor',
            name='type',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
    ]
