# Generated by Django 2.0.1 on 2018-01-23 15:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0002_auto_20180119_1210'),
    ]

    operations = [
        migrations.AddField(
            model_name='client',
            name='need_auth',
            field=models.IntegerField(blank=True, default=0, null=True),
        ),
        migrations.AddField(
            model_name='client',
            name='password',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='client',
            name='username',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='client',
            name='ip',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
    ]