# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2018-01-31 04:35
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0005_auto_20180131_1210'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='task',
            name='client',
        ),
        migrations.AddField(
            model_name='task',
            name='clients',
            field=models.TextField(blank=True, null=True),
        ),
    ]
