# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2018-07-11 15:32
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('core', '0008_auto_20180703_2305'),
    ]
    
    operations = [
        migrations.RemoveField(
            model_name='task',
            name='error',
        ),
        migrations.RemoveField(
            model_name='task',
            name='last',
        ),
        migrations.RemoveField(
            model_name='task',
            name='success',
        ),
        migrations.AddField(
            model_name='task',
            name='modified',
            field=models.BooleanField(default=False),
        ),
    ]
