# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2018-01-31 15:41
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0006_auto_20180131_1235'),
    ]

    operations = [
        migrations.AddField(
            model_name='task',
            name='trigger',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
    ]
